@@ -3,6 +3,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include <time.h>
 #include <gmp.h>
 
 #include "EXTERN.h"
@@ -11,16 +12,27 @@
 
 #include "gmp_main.h"
 #include "prime_iterator.h"
+#include "small_factor.h"
+#include "simpqs.h"
+#include "ecm.h"
+#define _GMP_ECM_FACTOR _GMP_ecm_factor_projective
+#include "utility.h"
 
 static int _verbose = 0;
 void _GMP_set_verbose(int v) { _verbose = v; }
 int _GMP_get_verbose(void) { return _verbose; }
 
 static gmp_randstate_t _randstate;
+gmp_randstate_t* _GMP_get_randstate(void) { return &_randstate; }
 
 void _GMP_init(void)
 {
+  /* We should  not use this random number system for crypto, so
+   * using this lousy seed is ok.  We just would like something a
+   * bit different every run. */
+  unsigned long seed = time(NULL);
   gmp_randinit_mt(_randstate);
+  gmp_randseed_ui(_randstate, seed);
   prime_iterator_global_startup();
 }
 
@@ -39,7 +51,7 @@ static const unsigned char wheel_advance[30] =
   {0,6,0,0,0,0,0,4,0,0,0,2,0,4,0,0,0,2,0,4,0,0,0,6,0,0,0,0,0,2};
 
 
-static int _GMP_miller_rabin_ui(mpz_t n, UV base)
+static INLINE int _GMP_miller_rabin_ui(mpz_t n, UV base)
 {
   int rval;
   mpz_t a;
@@ -63,13 +75,22 @@ int _GMP_miller_rabin(mpz_t n, mpz_t a)
   }
   mpz_init_set(nminus1, n);
   mpz_sub_ui(nminus1, nminus1, 1);
-  mpz_init_set(d, nminus1);
+  mpz_init_set(x, a);
 
+  /* Handle large and small bases.  Use x so we don't modify their input a. */
+  if (mpz_cmp(x, n) >= 0)
+    mpz_mod(x, x, n);
+  if ( (mpz_cmp_ui(x, 1) <= 0) || (mpz_cmp(x, nminus1) >= 0) ) {
+    mpz_clear(nminus1);
+    mpz_clear(x);
+    return 1;
+  }
+
+  mpz_init_set(d, nminus1);
   s = mpz_scan1(d, 0);
   mpz_tdiv_q_2exp(d, d, s);
 
-  mpz_init(x);
-  mpz_powm(x, a, d, n);
+  mpz_powm(x, x, d, n);
   mpz_clear(d); /* done with a and d */
   rval = 0;
   if (!mpz_cmp_ui(x, 1) || !mpz_cmp(x, nminus1)) {
@@ -113,15 +134,13 @@ int _GMP_is_strong_lucas_pseudoprime(mpz_t n)
     IV sign = 1;
     mpz_init(t);
     while (1) {
-      UV gcd, j;
-      gcd = mpz_gcd_ui(NULL, n, D_ui);
+      UV gcd = mpz_gcd_ui(NULL, n, D_ui);
       if ((gcd > 1) && mpz_cmp_ui(n, gcd) != 0) {
         D_ui = 0;
         break;
       }
       mpz_set_si(t, (IV)D_ui * sign);
-      j = mpz_jacobi(t, n);
-      if (j == -1)  break;
+      if (mpz_jacobi(t, n) == -1)  break;
       D_ui += 2;
       sign = -sign;
     }
@@ -131,7 +150,7 @@ int _GMP_is_strong_lucas_pseudoprime(mpz_t n)
   }
   Q = (1 - D) / 4;
   if (_verbose>3) gmp_printf("N: %Zd  D: %ld  P: %lu  Q: %ld\n", n, D, P, Q);
-  if (D != P*P - 4*Q)  croak("incorrect DPQ\n");
+  if (D != ((IV)(P*P)) - 4*Q)  croak("incorrect DPQ\n");
   /* Now start on the lucas sequence */
   mpz_init_set(d, n);
   mpz_add_ui(d, d, 1);
@@ -283,7 +302,7 @@ int _GMP_is_prime(mpz_t n)
    * to about 50 digits).
    */
   if (mpz_sizeinbase(n, 2) <= 200) {
-    prob_prime = _GMP_primality_bls(n, 1);
+    prob_prime = _GMP_primality_bls(n, 1, 0);
     if (prob_prime < 0)
       return 0;
     if (prob_prime > 0)
@@ -292,7 +311,7 @@ int _GMP_is_prime(mpz_t n)
   return 1;
 }
 
-int _GMP_is_provable_prime(mpz_t n)
+int _GMP_is_provable_prime(mpz_t n, char* prooftext)
 {
   int prob_prime;
 
@@ -311,7 +330,7 @@ int _GMP_is_provable_prime(mpz_t n)
      )
     return 0;
 
-  prob_prime = _GMP_primality_bls(n, 0);
+  prob_prime = _GMP_primality_bls(n, 0, &prooftext);
   if (prob_prime < 0)
     return 0;
   if (prob_prime > 0)
@@ -322,87 +341,10 @@ int _GMP_is_provable_prime(mpz_t n)
 /*****************************************************************************/
 /*          AKS.    This implementation is quite slow, but useful to have.   */
 
-static UV order(UV r, mpz_t n, UV limit) {
-  UV j;
-  mpz_t t;
-
-  mpz_init_set_ui(t, 1);
-  for (j = 1; j <= limit; j++) {
-    mpz_mul(t, t, n);
-    mpz_mod_ui(t, t, r);
-    if (!mpz_cmp_ui(t, 1))
-      break;
-  }
-  mpz_clear(t);
-  return j;
-}
-
-static void poly_mod_mul(mpz_t* px, mpz_t* py, mpz_t* ptmp, UV r, mpz_t mod)
-{
-  int i, j;
-  UV rindex;
-
-  for (i = 0; i < r; i++)
-    mpz_set_ui(ptmp[i], 0);
-  for (i = 0; i < r; i++) {
-    if (!mpz_sgn(px[i])) continue;
-    for (j = 0; j < r; j++) {
-      if (!mpz_sgn(py[j])) continue;
-      rindex = (i+j) % r;
-      mpz_addmul( ptmp[rindex], px[i], py[j] );
-    }
-  }
-  /* Put ptmp into px and mod n */
-  for (i = 0; i < r; i++)
-    mpz_mod(px[i], ptmp[i], mod);
-}
-static void poly_mod_sqr(mpz_t* px, mpz_t* ptmp, UV r, mpz_t mod)
-{
-  int i, d, s;
-  UV degree = r-1;
-
-  for (i = 0; i < r; i++)
-    mpz_set_ui(ptmp[i], 0);
-  for (d = 0; d <= 2*degree; d++) {
-    UV rindex = d % r;
-    for (s = (d <= degree) ? 0 : d-degree; s <= (d/2); s++) {
-      if (s*2 == d) {
-        mpz_addmul( ptmp[rindex], px[s], px[s] );
-      } else {
-        mpz_addmul( ptmp[rindex], px[s], px[d-s] );
-        mpz_addmul( ptmp[rindex], px[s], px[d-s] );
-      }
-    }
-  }
-  /* Put ptmp into px and mod n */
-  for (i = 0; i < r; i++)
-    mpz_mod(px[i], ptmp[i], mod);
-}
-
-static void poly_mod_pow(mpz_t *pres, mpz_t *pn, mpz_t *ptmp, mpz_t power, UV r, mpz_t mod)
-{
-  int i;
-  mpz_t pow;
-
-  for (i = 0; i < r; i++)
-    mpz_set_ui(pres[i], 0);
-  mpz_set_ui(pres[0], 1);
-
-  mpz_init_set(pow, power);
-
-  while (mpz_cmp_ui(pow, 0) > 0) {
-    if (mpz_odd_p(pow))            poly_mod_mul(pres, pn, ptmp, r, mod);
-    mpz_tdiv_q_2exp(pow, pow, 1);
-    if (mpz_cmp_ui(pow, 0) > 0)    poly_mod_sqr(pn, ptmp, r, mod);
-  }
-  mpz_clear(pow);
-}
-
 static int test_anr(UV a, mpz_t n, UV r, mpz_t* px, mpz_t* py, mpz_t* ptmp)
 {
-  int i;
   int retval = 1;
-  UV n_mod_r;
+  UV i, n_mod_r;
   mpz_t t;
 
   for (i = 0; i < r; i++)
@@ -433,8 +375,8 @@ int _GMP_is_aks_prime(mpz_t n)
 {
   mpz_t sqrtn;
   mpz_t *px, *py, *pt;
-  int i, retval;
-  UV log2n, limit, rlimit, r, a;
+  int retval;
+  UV i, log2n, limit, rlimit, r, a;
   PRIME_ITERATOR(iter);
 
   if (mpz_cmp_ui(n, 4) < 0) {
@@ -468,7 +410,7 @@ int _GMP_is_aks_prime(mpz_t n)
       mpz_clear(sqrtn);
       return 1;
     }
-    if (order(r, n, limit) > limit)
+    if (mpz_order_ui(r, n, limit) > limit)
       break;
   }
   prime_iterator_destroy(&iter);
@@ -539,9 +481,10 @@ int _GMP_is_aks_prime(mpz_t n)
  *
  * BLS: given n-1 = A*B, factored A, s=B/2A r=B mod (2A), and an a, then if:
  *   - A is even, B is odd, and AB=n-1 (all implied by n = odd and the above),
- *   - (A+1) * (2*A*A + (r-1) * A + 1) > n
- *   - a^(n-1) % n = 1
- *   - gcd(a^((n-1)/f)-1,n) = 1  for ALL factors f of A
+ *   - n < (A+1) * (2*A*A + (r-1) * A + 1)
+ *   - for each each factor f of A, there exists an a (1 < a < n-1) s.t.
+ *     - a^(n-1) % n = 1
+ *     - gcd(a^((n-1)/f)-1,n) = 1  for ALL factors f of A
  * then:
  *     if s = 0 or r*r - 8*s is not a perfect square
  *         n is prime
@@ -589,7 +532,7 @@ int _GMP_is_aks_prime(mpz_t n)
 #define primality_handle_factor(f, primality_func, factor_prob) \
   { \
     int f_prob_prime = _GMP_is_prob_prime(f); \
-    if ( (f_prob_prime == 1) && (primality_func(f, do_quick)) ) \
+    if ( (f_prob_prime == 1) && (primality_func(f, do_quick, prooftextptr)) ) \
       f_prob_prime = 2; \
     if (f_prob_prime == 2) { \
       if (fsp >= PRIM_STACK_SIZE) { success = 0; } \
@@ -666,10 +609,10 @@ int _GMP_primality_pocklington(mpz_t n, int do_quick)
       if (!success)  success = _GMP_pbrent_factor(m, f,11, 64*1024);
       if (!success)  success = _GMP_pbrent_factor(m, f,13, 64*1024);
       if (!success)  success = _GMP_pbrent_factor(m, f, 1, 16*1024*1024);
-      if (!success)  success = _GMP_ecm_factor   (m, f, 12500, 4);
-      if (!success)  success = _GMP_ecm_factor   (m, f, 3125000, 10);
+      if (!success)  success = _GMP_ECM_FACTOR   (m, f, 12500, 4);
+      if (!success)  success = _GMP_ECM_FACTOR   (m, f, 3125000, 10);
       if (!success)  success = _GMP_pbrent_factor(m, f, 3, 256*1024*1024);
-      if (!success)  success = _GMP_ecm_factor   (m, f, 400000000, 200);
+      if (!success)  success = _GMP_ECM_FACTOR   (m, f, 400000000, 200);
     }
     /* If we couldn't factor m and the stack is empty, we've failed. */
     if ( (!success) && (msp == 0) )
@@ -726,7 +669,7 @@ int _GMP_primality_pocklington(mpz_t n, int do_quick)
 }
 #endif
 
-int _GMP_primality_bls(mpz_t n, int do_quick)
+int _GMP_primality_bls(mpz_t n, int do_quick, char** prooftextptr)
 {
   mpz_t nm1, A, B, t, m, f, r, s;
   mpz_t mstack[PRIM_STACK_SIZE];
@@ -748,17 +691,22 @@ int _GMP_primality_bls(mpz_t n, int do_quick)
   mpz_init(r);
   mpz_init(s);
 
+#define TRIAL_B1 2000
   { /* Pull small factors out */
-    UV tf = 2;
-    while ( (tf = _GMP_trial_factor(B, tf, 1000)) != 0 ) {
-      if (fsp >= PRIM_STACK_SIZE) { success = 0; break; }
-      mpz_init_set_ui(fstack[fsp++], tf);
-      while (mpz_divisible_ui_p(B, tf)) {
-        mpz_mul_ui(A, A, tf);
-        mpz_divexact_ui(B, B, tf);
+    PRIME_ITERATOR(iter);
+    UV tf;
+    for (tf = 2; tf < TRIAL_B1; tf = prime_iterator_next(&iter)) {
+      if (mpz_cmp_ui(B, tf*tf) < 0) break;
+      if (mpz_divisible_ui_p(B, tf)) {
+        if (fsp >= PRIM_STACK_SIZE) { success = 0; break; }
+        mpz_init_set_ui(fstack[fsp++], tf);
+        do {
+          mpz_mul_ui(A, A, tf);
+          mpz_divexact_ui(B, B, tf);
+        } while (mpz_divisible_ui_p(B, tf));
       }
-      tf++;
     }
+    prime_iterator_destroy(&iter);
   }
 
   if (success) {
@@ -771,16 +719,31 @@ int _GMP_primality_bls(mpz_t n, int do_quick)
     mpz_tdiv_q(s, B, t);
     mpz_mod(r, B, t);
 
-    mpz_mul(m, t, A);    // m = 2*A*A
-    mpz_sub_ui(t, r, 1); // t = r-1
-    mpz_mul(t, t, A);    // t = A*(r-1)
-    mpz_add(m, m, t);    // m = 2A^2 + A(r-1)
-    mpz_add_ui(m, m, 1); // m = 2A^2 + A(r-1) + 1
+    mpz_mul(m, t, A);    /* m = 2*A*A              */
+    mpz_sub_ui(t, r, 1); /* t = r-1                */
+    mpz_mul(t, t, A);    /* t = A*(r-1)            */
+    mpz_add(m, m, t);    /* m = 2A^2 + A(r-1)      */
+    mpz_add_ui(m, m, 1); /* m = 2A^2 + A(r-1) + 1  */
     mpz_add_ui(t, A, 1);
     mpz_mul(m, m, t);
 
     if (mpz_cmp(m, n) > 0)
       break;
+#if 0
+    /* Theorem 7 would let us do this, though we also have to do one extra
+     * test at the end finding an a for B (treat the unfactored part B as if
+     * it was the last factor). */
+    mpz_mul(m, t, A);           /* m = 2*A*A               */
+    mpz_sub_ui(t, r, TRIAL_B1); /* t = r-B1                */
+    mpz_mul(t, t, A);           /* t = A*(r-B1)            */
+    mpz_add(m, m, t);           /* m = 2A^2 + A(r-B1)      */
+    mpz_add_ui(m, m, 1);        /* m = 2A^2 + A(r-B1) + 1  */
+    mpz_mul_ui(t, A, TRIAL_B1);
+    mpz_add_ui(t, t, 1);        /* t = B1*A+1              */
+    mpz_mul(m, m, t);           /* m = (B1*A+1)*(2A^2+(r-B1)A+1) */
+    if (mpz_cmp(m, n) > 0)
+      break;
+#endif
     success = 0;
     /* If the stack is empty, we have failed. */
     if (msp == 0)
@@ -790,50 +753,104 @@ int _GMP_primality_bls(mpz_t n, int do_quick)
 
     /* Try to factor it without trying too hard */
     if (!success)  success = _GMP_power_factor(m, f);
+
+    if (!success && mpz_cmp_ui(m, (unsigned long)(UV_MAX>>2)) < 0) {
+      UV ui_m = mpz_get_ui(m);
+      UV ui_factors[2];
+      if (!mpz_cmp_ui(m, ui_m)) {
+        success = racing_squfof_factor(ui_m, ui_factors, 200000)-1;
+        if (success)
+          mpz_set_ui(f, ui_factors[0]);
+      }
+    }
     if (do_quick) {
+      /* These keep the time in the realm of 2ms per input number, which
+       * means they adjust for size.  Success varies by size:
+       *   99% of  70-bit inputs
+       *   90% of  90-bit inputs
+       *   58% of 120-bit inputs
+       *   25% of 160-bit inputs
+       *   10% of 190-bit inputs
+       */
       UV log2m = mpz_sizeinbase(m, 2);
-      UV rounds = (log2m <= 64) ? 300000 : 300000 / (log2m-63);
-      if (!success)  success = _GMP_pbrent_factor(m, f, 3, rounds);
+      UV brent_rounds = (log2m <= 64) ? 100000 : 100000 / (log2m-63);
+      int final_B2 = 1000 * (150-(int)log2m);
+      if (log2m < 70) brent_rounds *= 3;
+      if (!success)  success = _GMP_pminus1_factor(m, f, 100, 1000);
+      if (!success)  success = _GMP_pminus1_factor(m, f, 1000, 10000);
+      if (!success && log2m < 80)  success = _GMP_ECM_FACTOR(m, f, 150, 5);
+      if (!success)  success = _GMP_pbrent_factor(m, f, 3, brent_rounds);
+      if (!success && final_B2 > 10000)  success = _GMP_pminus1_factor(m, f, 10000, final_B2);
     } else {
-      if (!success)  success = _GMP_pbrent_factor(m, f, 3, 64*1024);
-      if (!success)  success = _GMP_pbrent_factor(m, f, 5, 64*1024);
-      if (!success)  success = _GMP_pbrent_factor(m, f, 7, 64*1024);
-      if (!success)  success = _GMP_pbrent_factor(m, f,11, 64*1024);
-      if (!success)  success = _GMP_pbrent_factor(m, f,13, 64*1024);
-      if (!success)  success = _GMP_pminus1_factor(m, f, 100000, 1000000);
-      if (!success)  success = _GMP_ecm_factor   (m, f, 125000, 5);
-      if (!success)  success = _GMP_pbrent_factor(m, f, 1, 8*1024*1024);
-      if (!success)  success = _GMP_ecm_factor   (m, f, 3125000, 10);
-      if (!success)  success = _GMP_pbrent_factor(m, f, 3, 256*1024*1024);
-      if (!success)  success = _GMP_ecm_factor   (m, f, 400000000, 200);
+      if (!success)  success = _GMP_pminus1_factor(m, f, 100, 1000);
+      if (!success)  success = _GMP_pminus1_factor(m, f, 1000, 10000);
+      if (!success)  success = _GMP_pminus1_factor(m, f, 10000, 200000);
+      if (!success)  success = _GMP_ECM_FACTOR(m, f, 500, 30);
+      if (!success)  success = _GMP_ECM_FACTOR(m, f, 2000, 20);
+      if (!success)  success = _GMP_pminus1_factor(m, f, 200000, 4000000);
+      if (!success)  success = _GMP_ECM_FACTOR(m, f, 10000, 10);
+      /* QS.  Uses lots of memory, but finds multiple factors quickly */
+      if (!success && mpz_sizeinbase(m,10)>=30 && mpz_sizeinbase(m,2)<300) {
+            mpz_t farray[66];
+            int i, nfactors;
+            for (i = 0; i < 66; i++)
+              mpz_init(farray[i]);
+            nfactors = _GMP_simpqs(m, farray);
+            /* Insert all found factors */
+            if (nfactors > 1) {
+              success = 1;
+              for (i = 0; i < nfactors; i++) {
+                primality_handle_factor(farray[i], _GMP_primality_bls, 0);
+              }
+            }
+            for (i = 0; i < 66; i++)
+              mpz_clear(farray[i]);
+            if (success)
+              continue;
+      }
+      if (!success) {
+        UV i;
+        UV B1 = 10000;
+        UV curves = 10;
+        for (i = 1; i < 18; i++) {
+          B1 *= 2;
+          success = _GMP_ECM_FACTOR(m, f, B1, curves);
+          if (success) break;
+        }
+      }
     }
     /* If we couldn't factor m and the stack is empty, we've failed. */
     if ( (!success) && (msp == 0) )
       break;
-    /* Put the two factors f and m/f into the stacks */
-    primality_handle_factor(f, _GMP_primality_bls, 0);
-    mpz_divexact(f, m, f);
+    /* Put the two factors f and m/f into the stacks, smallest first */
+    mpz_divexact(m, m, f);
+    if (mpz_cmp(m, f) < 0) { mpz_set(t, m); mpz_set(m, f); mpz_set(f, t); }
     primality_handle_factor(f, _GMP_primality_bls, 0);
+    primality_handle_factor(m, _GMP_primality_bls, 0);
   }
 
-  if (success) {
-    int pcount, a, proper_r;
+  /* clear mstack since we don't care about it.  Use to hold a values. */
+  while (msp-- > 0)
+    mpz_clear(mstack[msp]);
+  msp = 0;
+
+  if (success > 0) {
+    mpz_mul(t, r, r);
+    mpz_submul_ui(t, s, 8);   /* t = r^2 - 8s */
+    /* N is prime if and only if s=0 OR t not a perfect square */
+    /* So, N is a composite if s != 0 and t is perfect square */
+    if (mpz_sgn(s) != 0 && mpz_perfect_square_p(t))
+      success = -1;
+  }
+
+  if (success > 0) {
+    int pcount, a;
     int const alimit = do_quick ? 200 : 10000;
     mpz_t p, ap;
 
     mpz_init(p);
     mpz_init(ap);
 
-    proper_r = 0;
-    if (!mpz_sgn(s)) {
-      proper_r = 1;
-    } else {
-      mpz_mul(t, r, r);
-      mpz_submul_ui(t, s, 8);   // t = r^2 - 8s
-      if ( ! mpz_perfect_square_p(t) )
-        proper_r = 1;
-    }
-
     for (pcount = 0; success && pcount < fsp; pcount++) {
       PRIME_ITERATOR(iter);
       mpz_set(p, fstack[pcount]);
@@ -852,22 +869,28 @@ int _GMP_primality_bls(mpz_t n, int do_quick)
         if (mpz_cmp_ui(t, 1) != 0)
           continue;
         success = 1;   /* We found an a for this p */
+        mpz_init_set(mstack[msp++], ap);
       }
       prime_iterator_destroy(&iter);
     }
-
-    /* If all cases hold then n is prime if and only if proper_r */
-    if ( success && !proper_r )
-      success = -1;
     mpz_clear(p);
     mpz_clear(ap);
   }
-  while (msp-- > 0) {
-    mpz_clear(mstack[msp]);
+  if (success > 0 && prooftextptr != 0 && *prooftextptr != 0) {
+    int i;
+    if (fsp != msp) croak("Different f and a counts\n");
+    *prooftextptr += gmp_sprintf(*prooftextptr, "%Zd :", n);
+    for (i = 0; i < fsp; i++)
+      *prooftextptr += gmp_sprintf(*prooftextptr, " %Zd", fstack[i]);
+    *prooftextptr += gmp_sprintf(*prooftextptr, " :");
+    for (i = 0; i < msp; i++)
+      *prooftextptr += gmp_sprintf(*prooftextptr, " %Zd", mstack[i]);
+    *prooftextptr += gmp_sprintf(*prooftextptr, "\n");
   }
-  while (fsp-- > 0) {
+  while (fsp-- > 0)
     mpz_clear(fstack[fsp]);
-  }
+  while (msp-- > 0)
+    mpz_clear(mstack[msp]);
   mpz_clear(nm1);
   mpz_clear(A);
   mpz_clear(B);
@@ -993,12 +1016,12 @@ int _GMP_prho_factor(mpz_t n, mpz_t f, UV a, UV rounds)
   mpz_init(oldV);
   while (rounds-- > 0) {
     mpz_set_ui(m, 1); mpz_set(oldU, U);  mpz_set(oldV, V);
-    for (i = 0; i < inner; i++) {
+    for (i = 0; i < (int)inner; i++) {
       mpz_mul(U, U, U);  mpz_add_ui(U, U, a);  mpz_tdiv_r(U, U, n);
       mpz_mul(V, V, V);  mpz_add_ui(V, V, a);  mpz_tdiv_r(V, V, n);
       mpz_mul(V, V, V);  mpz_add_ui(V, V, a);  mpz_tdiv_r(V, V, n);
-      mpz_sub(f, U, V);
-      if (mpz_sgn(f) < 0)  mpz_add(f, f, n);
+      if (mpz_cmp(U, V) >= 0)  mpz_sub(f, U, V);
+      else                     mpz_sub(f, V, U);
       mpz_mul(m, m, f);
       mpz_tdiv_r(m, m, n);
     }
@@ -1013,8 +1036,8 @@ int _GMP_prho_factor(mpz_t n, mpz_t f, UV a, UV rounds)
         mpz_mul(U, U, U);  mpz_add_ui(U, U, a);  mpz_tdiv_r(U, U, n);
         mpz_mul(V, V, V);  mpz_add_ui(V, V, a);  mpz_tdiv_r(V, V, n);
         mpz_mul(V, V, V);  mpz_add_ui(V, V, a);  mpz_tdiv_r(V, V, n);
-        mpz_sub(f, U, V);
-        if (mpz_sgn(f) < 0)  mpz_add(f, f, n);
+        if (mpz_cmp(U, V) >= 0)  mpz_sub(f, U, V);
+        else                     mpz_sub(f, V, U);
         mpz_gcd(f, f, n);
       } while (!mpz_cmp_ui(f, 1) && i-- != 0);
       if ( (!mpz_cmp_ui(f, 1)) || (!mpz_cmp(f, n)) )  break;
@@ -1029,7 +1052,7 @@ int _GMP_prho_factor(mpz_t n, mpz_t f, UV a, UV rounds)
 
 int _GMP_pbrent_factor(mpz_t n, mpz_t f, UV a, UV rounds)
 {
-  mpz_t Xi, Xm, saveXi, m;
+  mpz_t Xi, Xm, saveXi, m, t;
   UV i, r;
   const UV inner = 256;
 
@@ -1037,6 +1060,7 @@ int _GMP_pbrent_factor(mpz_t n, mpz_t f, UV a, UV rounds)
   mpz_init_set_ui(Xi, 2);
   mpz_init_set_ui(Xm, 2);
   mpz_init(m);
+  mpz_init(t);
   mpz_init(saveXi);
 
   r = 1;
@@ -1047,11 +1071,11 @@ int _GMP_pbrent_factor(mpz_t n, mpz_t f, UV a, UV rounds)
       mpz_set_ui(m, 1);
       mpz_set(saveXi, Xi);
       for (i = 0; i < dorounds; i++) {
-        mpz_mul(Xi, Xi, Xi);  mpz_add_ui(Xi, Xi, a);  mpz_tdiv_r(Xi, Xi, n);
-        mpz_sub(f, Xi, Xm);
-        if (mpz_sgn(f) < 0)  mpz_add(f, f, n);
-        mpz_mul(m, m, f);
-        mpz_tdiv_r(m, m, n);
+        mpz_mul(t, Xi, Xi);  mpz_add_ui(t, t, a);  mpz_tdiv_r(Xi, t, n);
+        if (mpz_cmp(Xi, Xm) >= 0)  mpz_sub(f, Xi, Xm);
+        else                       mpz_sub(f, Xm, Xi);
+        mpz_mul(t, m, f);
+        mpz_tdiv_r(m, t, n);
       }
       rleft -= dorounds;
       rounds -= dorounds;
@@ -1068,17 +1092,17 @@ int _GMP_pbrent_factor(mpz_t n, mpz_t f, UV a, UV rounds)
       /* f == n, so we have to back up to see what factor got found */
       mpz_set(Xi, saveXi);
       do {
-        mpz_mul(Xi, Xi, Xi);  mpz_add_ui(Xi, Xi, a);  mpz_tdiv_r(Xi, Xi, n);
-        mpz_sub(f, Xi, Xm);
-        if (mpz_sgn(f) < 0)  mpz_add(f, f, n);
+        mpz_mul(t, Xi, Xi);  mpz_add_ui(t, t, a);  mpz_tdiv_r(Xi, t, n);
+        if (mpz_cmp(Xi, Xm) >= 0)  mpz_sub(f, Xi, Xm);
+        else                       mpz_sub(f, Xm, Xi);
         mpz_gcd(f, f, n);
       } while (!mpz_cmp_ui(f, 1) && r-- != 0);
       if ( (!mpz_cmp_ui(f, 1)) || (!mpz_cmp(f, n)) )  break;
     }
-    mpz_clear(Xi); mpz_clear(Xm); mpz_clear(m); mpz_clear(saveXi);
+    mpz_clear(Xi); mpz_clear(Xm); mpz_clear(m); mpz_clear(saveXi); mpz_clear(t);
     return 1;
   }
-  mpz_clear(Xi); mpz_clear(Xm); mpz_clear(m); mpz_clear(saveXi);
+  mpz_clear(Xi); mpz_clear(Xm); mpz_clear(m); mpz_clear(saveXi); mpz_clear(t);
   mpz_set(f, n);
   return 0;
 }
@@ -1120,7 +1144,7 @@ void _GMP_lcm_of_consecutive_integers(UV B, mpz_t m)
 
 int _GMP_pminus1_factor(mpz_t n, mpz_t f, UV B1, UV B2)
 {
-  mpz_t a, savea, b, t;
+  mpz_t a, savea, t;
   UV q, saveq, j;
   PRIME_ITERATOR(iter);
 
@@ -1129,10 +1153,9 @@ int _GMP_pminus1_factor(mpz_t n, mpz_t f, UV B1, UV B2)
 
   mpz_init(a);
   mpz_init(savea);
-  mpz_init(b);
   mpz_init(t);
 
-  if (_verbose>2) gmp_printf("# trying %Zd  with B=%lu\n", n, (unsigned long)B1);
+  if (_verbose>2) gmp_printf("# p-1 trying %Zd (B1=%lu B2=%lu)\n", n, (unsigned long)B1, (unsigned long)B2);
 
   /* STAGE 1
    * Montgomery 1987 p249-250 and Brent 1990 p5 both indicate we can calculate
@@ -1226,7 +1249,6 @@ int _GMP_pminus1_factor(mpz_t n, mpz_t f, UV B1, UV B2)
     mpz_t precomp_bm[111];
     int   is_precomp[111] = {0};
 
-    if (_verbose>2) gmp_printf("# Starting second stage from %lu to %lu\n", (unsigned long)B1, (unsigned long)B2);
     mpz_init(bmdiff);
     mpz_init_set(bm, a);
     mpz_init_set_ui(b, 1);
@@ -1261,13 +1283,13 @@ int _GMP_pminus1_factor(mpz_t n, mpz_t f, UV B1, UV B2)
         mpz_init_set(precomp_bm[qdiff], bmdiff);
         is_precomp[qdiff] = 1;
       }
-      mpz_mul(a, a, bmdiff);
-      mpz_tdiv_r(a, a, n);
+      mpz_mul(t, a, bmdiff);
+      mpz_tdiv_r(a, t, n);
       if ( !mpz_sgn(a) )
         break;
       mpz_sub_ui(t, a, 1);
-      mpz_mul(b, b, t);
-      mpz_tdiv_r(b, b, n);
+      mpz_mul(t, b, t);
+      mpz_tdiv_r(b, t, n);
       if ( (j++ % 64) == 0) {     /* GCD every so often */
         mpz_gcd(f, b, n);
         if ( (mpz_cmp_ui(f, 1) != 0) && (mpz_cmp(f, n) != 0) )
@@ -1282,7 +1304,6 @@ int _GMP_pminus1_factor(mpz_t n, mpz_t f, UV B1, UV B2)
       if (is_precomp[j])
         mpz_clear(precomp_bm[j]);
     }
-    if (_verbose>2) gmp_printf("# End second stage\n");
     if ( (mpz_cmp_ui(f, 1) != 0) && (mpz_cmp(f, n) != 0) )
       goto end_success;
   }
@@ -1293,10 +1314,12 @@ int _GMP_pminus1_factor(mpz_t n, mpz_t f, UV B1, UV B2)
     prime_iterator_destroy(&iter);
     mpz_clear(a);
     mpz_clear(savea);
-    mpz_clear(b);
     mpz_clear(t);
-    if ( (mpz_cmp_ui(f, 1) != 0) && (mpz_cmp(f, n) != 0) )
+    if ( (mpz_cmp_ui(f, 1) != 0) && (mpz_cmp(f, n) != 0) ) {
+      if (_verbose>2) gmp_printf("# p-1: %Zd\n", f);
       return 1;
+    }
+    if (_verbose>2) gmp_printf("# p-1: no factor\n");
     mpz_set(f, n);
     return 0;
 }
@@ -1561,191 +1584,3 @@ int _GMP_power_factor(mpz_t n, mpz_t f)
   }
   return 0;
 }
-
-struct _ec_point { mpz_t x, y; };
-
-/* P3 = P1 + P2 */
-static void _ec_add_AB(mpz_t n,
-                    struct _ec_point P1,
-                    struct _ec_point P2,
-                    struct _ec_point *P3,
-                    mpz_t m,
-                    mpz_t t1,
-                    mpz_t t2)
-{
-  if (!mpz_cmp(P1.x, P2.x)) {
-    mpz_add(t2, P1.y, P2.y);
-    mpz_mod(t1, t2, n);
-    if (!mpz_cmp_ui(t1, 0) ) {
-      mpz_set_ui(P3->x, 0);
-      mpz_set_ui(P3->y, 1);
-      return;
-    }
-  }
-
-  mpz_sub(t1, P2.x, P1.x);
-  mpz_mod(t2, t1, n);
-
-  /* m = (y2 - y1) * (x2 - x1)^-1 mod n */
-  if (!mpz_invert(t1, t2, n)) {
-    /* There are ways we should use to handle this, but for now just bail. */
-    mpz_set_ui(P3->x, 0);
-    mpz_set_ui(P3->y, 1);
-    return;
-  }
-
-  mpz_sub(m, P2.y, P1.y);
-  mpz_mod(t2, m, n);        /* t2 = deltay */
-  mpz_mul(m, t1, t2);
-  mpz_mod(m, m, n);         /* m = deltay / deltax */
-
-  /* x3 = m^2 - x1 - x2 mod n */
-  mpz_mul(t1, m, m);
-  mpz_sub(t2, t1, P1.x);
-  mpz_sub(t1, t2, P2.x);
-  mpz_mod(P3->x, t1, n);
-  /* y3 = m(x1 - x3) - y1 mod n */
-  mpz_sub(t1, P1.x, P3->x);
-  mpz_mul(t2, m, t1);
-  mpz_sub(t1, t2, P1.y);
-  mpz_mod(P3->y, t1, n);
-}
-
-/* P3 = 2*P1 */
-static void _ec_add_2A(mpz_t a,
-                    mpz_t n,
-                    struct _ec_point P1,
-                    struct _ec_point *P3,
-                    mpz_t m,
-                    mpz_t t1,
-                    mpz_t t2)
-{
-  /* m = (3x1^2 + a) * (2y1)^-1 mod n */
-  mpz_mul_ui(t1, P1.y, 2);
-  if (!mpz_invert(m, t1, n)) {
-    mpz_set_ui(P3->x, 0);
-    mpz_set_ui(P3->y, 1);
-    return;
-  }
-  mpz_mul_ui(t1, P1.x, 3);
-  mpz_mul(t2, t1, P1.x);
-  mpz_add(t1, t2, a);
-  mpz_mul(t2, m, t1);
-  mpz_tdiv_r(m, t2, n);
-
-  /* x3 = m^2 - 2x1 mod n */
-  mpz_mul(t1, m, m);
-  mpz_mul_ui(t2, P1.x, 2);
-  mpz_sub(t1, t1, t2);
-  mpz_tdiv_r(P3->x, t1, n);
-
-  /* y3 = m(x1 - x3) - y1 mod n */
-  mpz_sub(t1, P1.x, P3->x);
-  mpz_mul(t2, t1, m);
-  mpz_sub(t1, t2, P1.y);
-  mpz_tdiv_r(P3->y, t1, n);
-}
-
-static int _ec_multiply(mpz_t a, UV k, mpz_t n, struct _ec_point P, struct _ec_point *R, mpz_t d)
-{
-  int found = 0;
-  struct _ec_point A, B, C;
-  mpz_t t, t2, t3, mult;
-
-  mpz_init(A.x); mpz_init(A.y);
-  mpz_init(B.x); mpz_init(B.y);
-  mpz_init(C.x); mpz_init(C.y);
-  mpz_init(t);   mpz_init(t2);   mpz_init(t3);
-  mpz_init_set_ui(mult, 1);  /* holds intermediates, gcd at end */
-
-  mpz_set(A.x, P.x);  mpz_set(A.y, P.y);
-  mpz_set_ui(B.x, 0); mpz_set_ui(B.y, 1);
-
-  /* Binary ladder multiply.  Should investigate Lucas chains. */
-  while (k > 0) {
-    if ( k & 1 ) {
-      mpz_sub(t, B.x, A.x);
-      mpz_mul(t2, mult, t);
-      mpz_mod(mult, t2, n);
-
-      if ( !mpz_cmp_ui(A.x, 0) && !mpz_cmp_ui(A.y, 1) ) {
-        /* nothing */
-      } else if ( !mpz_cmp_ui(B.x, 0) && !mpz_cmp_ui(B.y, 1) ) {
-        mpz_set(B.x, A.x);  mpz_set(B.y, A.y);
-      } else {
-        _ec_add_AB(n, A, B, &C, t, t2, t3);
-        mpz_set(B.x, C.x);  mpz_set(B.y, C.y);
-      }
-      k--;
-    } else {
-      mpz_mul_ui(t, A.y, 2);
-      mpz_mul(t2, mult, t);
-      mpz_mod(mult, t2, n);
-
-      _ec_add_2A(a, n, A, &C, t, t2, t3);
-      mpz_set(A.x, C.x);  mpz_set(A.y, C.y);
-      k >>= 1;
-    }
-  }
-  mpz_gcd(d, mult, n);
-  found = (mpz_cmp_ui(d, 1) && mpz_cmp(d, n));
-
-  mpz_tdiv_r(R->x, B.x, n);
-  mpz_tdiv_r(R->y, B.y, n);
-
-  mpz_clear(mult);
-  mpz_clear(t);   mpz_clear(t2);   mpz_clear(t3);
-  mpz_clear(A.x); mpz_clear(A.y);
-  mpz_clear(B.x); mpz_clear(B.y);
-  mpz_clear(C.x); mpz_clear(C.y);
-
-  return found;
-}
-
-int _GMP_ecm_factor(mpz_t n, mpz_t f, UV B1, UV ncurves)
-{
-  mpz_t a;
-  struct _ec_point X, Y;
-  UV B, curve, q;
-
-  TEST_FOR_2357(n, f);
-
-  mpz_init(a);
-  mpz_init(X.x); mpz_init(X.y);
-  mpz_init(Y.x); mpz_init(Y.y);
-
-  for (B = 100; B < B1*5; B *= 5) {
-    if (B*5 > 2*B1) B = B1;
-    for (curve = 0; curve < ncurves; curve++) {
-      PRIME_ITERATOR(iter);
-      mpz_urandomm(a, _randstate, n);
-      mpz_set_ui(X.x, 0); mpz_set_ui(X.y, 1);
-      for (q = 2; q < B; q = prime_iterator_next(&iter)) {
-        UV k = q;
-        UV kmin = B / q;
-
-        while (k <= kmin)
-          k *= q;
-
-        if (_ec_multiply(a, k, n, X, &Y, f)) {
-          prime_iterator_destroy(&iter);
-          mpz_clear(a);
-          mpz_clear(X.x); mpz_clear(X.y);
-          mpz_clear(Y.x); mpz_clear(Y.y);
-          return 1;
-        }
-        mpz_set(X.x, Y.x);  mpz_set(X.y, Y.y);
-        /* Check that we're not starting over */
-        if ( !mpz_cmp_ui(X.x, 0) && !mpz_cmp_ui(X.y, 1) )
-          break;
-      }
-      prime_iterator_destroy(&iter);
-    }
-  }
-
-  mpz_clear(a);
-  mpz_clear(X.x); mpz_clear(X.y);
-  mpz_clear(Y.x); mpz_clear(Y.y);
-
-  return 0;
-}
